@@ -2,11 +2,20 @@
 from .base import Base
 from pathlib import Path
 from functools import partial
+import datetime
+import tempfile
+import requests
 from .utils import build_model, attempt_to_import
 import tqdm
 import time
+import re
 
 altair = attempt_to_import('altair')
+nib = attempt_to_import('nibabel')
+nilearn = attempt_to_import('nilearn')
+
+
+TMP_DIR = Path(tempfile.mkdtemp())
 
 
 class Analysis:
@@ -18,6 +27,7 @@ class Analysis:
 
     _aliased_methods_ = ['delete', 'get_bundle', 'compile', 'generate_report',
                          'get_report', 'upload_neurovault', 'get_uploads',
+                         'load_uploads', 'plot_uploads',
                          'plot_report', 'get_design_matrix']
 
     def __init__(self, *, analyses, name, dataset_id, **kwargs):
@@ -160,7 +170,7 @@ class Analyses(Base):
         return self.post(id=id, sub_route='clone')
 
     def create_analysis(self, *, name, dataset_name, predictor_names,
-                        task=None, subject=None, run=None, session=None,
+                        tasks=None, subjects=None, runs=None, session=None,
                         hrf_variables=None, contrasts=None,
                         dummy_contrasts=True, transformations=None, **kwargs):
         """ Analysis creation "wizard". Given run selection filters, and name
@@ -177,36 +187,38 @@ class Analyses(Base):
             dataset = dataset[0]
 
         # Get task name
-        if task is not None:
-            search = [t for t in dataset['tasks'] if t['name'] == task]
-            if len(search) != 1:
-                raise ValueError(
-                    "Task name does not match any tasks in the dataset")
-            task_id = search[0]['id']
+        if tasks is not None:
+            if not isinstance(tasks, list):
+                tasks = [tasks]
+
+            task_ids = []
+            for task in tasks:
+                search = [t for t in dataset['tasks'] if t['name'] == task]
+                if len(search) != 1:
+                    raise ValueError(
+                        "Task name does not match any tasks in the dataset")
+                task_ids.append(search[0]['id'])
         else:
-            if len(dataset['tasks']) > 1:
-                raise ValueError(
-                    "No task specified, but dataset has more than one task")
-            res = dataset['tasks'][0]
-            task = res['name']
-            task_id = res['id']
+            # All tasks
+            tasks = [t['name'] for t in dataset['tasks']]
+            task_ids = [t['id'] for t in dataset['tasks']]
 
         # Get Run IDs
         run_models = self._client.runs.get(
-            dataset_id=dataset['id'], task_id=task_id,
-            subject=subject, number=run, session=session)
+            dataset_id=dataset['id'], task_id=task_ids,
+            subject=subjects, number=runs, session=session)
 
         if len(run_models) < 1:
             raise ValueError("No runs could be found with the given criterion")
 
-        subject = list(set(r['subject'] for r in run_models))
-        run = list(set(r['number'] for r in run_models if r['number']))
-        run = run or None
+        subjects = list(set(r['subject'] for r in run_models))
+        runs = list(set(r['number'] for r in run_models if r['number']))
+        runs = runs or None
 
-        run_id = [r['id'] for r in run_models]
+        run_ids = [r['id'] for r in run_models]
         # Get Predictor IDs
         public_preds = self._client.predictors.get(
-            run_id=run_id, name=predictor_names, active_only=False)
+            run_id=run_ids, name=predictor_names, active_only=False)
 
         predictors = [p['id'] for p in public_preds]
 
@@ -218,7 +230,7 @@ class Analyses(Base):
             for pred in private_preds:
                 predictors += [p['id']
                                for p in self._client.user.get_predictors(
-                                   run_id=run_id, name=pred)]
+                                   run_id=run_ids, name=pred)]
 
         if len(predictors) != len(predictor_names):
             raise ValueError(
@@ -229,15 +241,15 @@ class Analyses(Base):
         if transformations:
             transformations = transformations.copy()
         model = build_model(
-            name, predictor_names, task,
-            subject=subject, run=run, session=session,
+            name, predictor_names, tasks,
+            subjects=subjects, runs=runs, session=session,
             hrf_variables=hrf_variables,
             transformations=transformations,
             contrasts=contrasts, dummy_contrasts=dummy_contrasts
             )
 
         analysis = Analysis(analyses=self, dataset_id=dataset['id'],
-                            name=name, model=model, runs=run_id,
+                            name=name, model=model, runs=run_ids,
                             predictors=predictors, **kwargs)
 
         return analysis
@@ -316,7 +328,7 @@ class Analyses(Base):
     def upload_neurovault(self, id, validation_hash, subject_paths=None,
                           group_paths=None, collection_id=None, force=False,
                           cli_version=None, fmriprep_version=None,
-                          n_subjects=None):
+                          estimator=None, n_subjects=None):
         """ Submit analysis for report generation
         :param str id: Analysis hash_id.
         :param str validation_hash: Validation hash string.
@@ -325,6 +337,7 @@ class Analyses(Base):
         :param bool force: Force upload with unique timestamped name.
         :param str: neuroscout-cli version at runtime
         :param str: fmriprep version at runtime
+        :param str: estimator used in fitlins (anfi/nilearn)
         :param int n_subjects: Number of subjects in analysis.
         :return: client response object
         """
@@ -345,8 +358,9 @@ class Analyses(Base):
                 req = self.post(
                     id=id, sub_route='upload', files=files, level='GROUP',
                     validation_hash=validation_hash, force=force,
-                    fmriprep_version=fmriprep_version, cli_version=cli_version,
-                    n_subjects=n_subjects, collection_id=collection_id)
+                    fmriprep_version=fmriprep_version, estimator=estimator,
+                    cli_version=cli_version, n_subjects=n_subjects,
+                    collection_id=collection_id)
                 if collection_id is None:
                     collection_id = req['collection_id']
 
@@ -357,8 +371,8 @@ class Analyses(Base):
                 req = self.post(
                     id=id, sub_route='upload', files=files, level='SUBJECT',
                     validation_hash=validation_hash, force=force,
-                    fmriprep_version=fmriprep_version, cli_version=cli_version,
-                    collection_id=collection_id)
+                    fmriprep_version=fmriprep_version, estimator=estimator,
+                    cli_version=cli_version, collection_id=collection_id)
                 if collection_id is None:
                     collection_id = req['collection_id']
 
@@ -374,6 +388,102 @@ class Analyses(Base):
         """
         return self.get(id=id, sub_route='upload')
 
+    def load_uploads(self, id, select='latest',
+                     download_dir=None, **kwargs):
+        """ Load collection upload as NiBabel images and associated meta-data
+        You can filter which images are loaded based on either collection
+        level attributes or statmap image level attributes. These correspond
+        to field returns for `get_uploads` at the collection level or
+        `file` level. In addition for images, BIDS entities are parsed
+        and available to filter on.
+
+        :param str id: Analysis hash_id.
+        :param str select: How to select from multiple collections.
+        Options: "latest", "oldest" or None. If None, returns all results.
+        :param str download_dir: Path to download images. If None, tempdir.
+        :param dict kwargs: Attributes to filter images on.
+        If any attributes are not found, they are ignored.
+        :return list list of tuples of format (Nifti1Image, kwargs).
+        """
+        if download_dir is None:
+            download_dir = TMP_DIR
+        else:
+            download_dir = Path(download_dir)
+
+        # Sort uploads for upload date
+        uploads = self.get_uploads(id)
+        for u in uploads:
+            u['uploaded_at'] = datetime.datetime.strptime(
+                u['uploaded_at'], '%Y-%m-%dT%H:%M')
+        uploads = sorted(uploads, key=lambda x: x['uploaded_at'],
+                         reverse=(select == 'latest'))
+
+        # Select collections based on filters
+        uploads = [
+            u for u in uploads
+            if all([u.get(k) == v for k, v in kwargs.items() if k in u])
+            ]
+
+        if not uploads:
+            return None
+
+        # Select first item unless all are requested
+        if select is not None:
+            uploads = [uploads[0]]
+
+        # Extract entities from file path
+        def _get_entities(path):
+            di = {}
+            for t in ['task', 'contrast', 'stat']:
+                matches = re.findall(f"{t}-(.*?)_", path)
+                if matches:
+                    di[t] = matches[0]
+            return di
+
+        # Filter files, download if necessary and load as Niimg-object
+        flat = []
+        for u in uploads:
+            for f in u.pop('files'):
+                f = {**f, **_get_entities(f['basename'])}
+
+                # If file matches kwargs and is in NV
+                if f.pop('status') == 'OK' and all(
+                  [f.get(k, None) == v for k, v in kwargs.items() if k in f]):
+                    # Download and open
+                    img_url = "https://neurovault.org/media/images/" \
+                        f"{u['collection_id']}/{f['basename']}"
+                    f_name = download_dir / \
+                        f"{u['collection_id']}_{f['basename']}"
+
+                    if not f_name.exists():
+                        print(".", end ="")  
+                        with f_name.open('wb') as file:
+                            file.write(requests.get(img_url).content)
+                    niimg = nib.load(f_name)
+
+                    f.pop('traceback')
+                    flat.append((niimg, {**u, **f}))
+        return flat
+
+    def plot_uploads(self, id, plot_args={}, **kwargs):
+        """ Plot uploads for matching collections using nilearn
+        :param str id: Analysis hash_id.
+        :param dict kwargs: Arguments for load_uploads
+        :return list list of matplotlib objects.
+        """
+
+        images = self.load_uploads(id, **kwargs)
+
+        if images:
+            plots = []
+            for niimg, _ in images:
+                plots.append(
+                    nilearn.plotting.plot_stat_map(niimg, **plot_args))
+
+            return plots
+        else:
+            return None
+
     def get_full(self, id):
         """ Get full analysis object (including runs and predictors)
         :param str id: Analysis hash_id.
